/*
 * camera class, holds all data about camera

 */


#include <stdlib.h> 
#include <stdio.h>
#include <GL/glew.h>
#include <GL/glut.h>
#include <cuda_runtime_api.h>
#include <cuda_gl_interop.h>
#include <cutil.h>
#include <math.h>
#include "Util.h"

#include "Camera.h"


Camera::Camera() {
	mPos.x = -5;
	mPos.y = 5;
	mPos.z = -20;

	mPos.x = 0;
	mPos.y = 0;
	mPos.z = -50;

	mLookAt.x = 0;
	mLookAt.y = 0;
	mLookAt.z = 0;

	mUp.x = 0;
	mUp.y = 1;
	mUp.z = 0;
	
	float3 forward;
	forward.x = mLookAt.x - mPos.x;
	forward.y = mLookAt.y - mPos.y;
	forward.z = mLookAt.z - mPos.z;
	normalize(forward.x, forward.y, forward.z);


	// lookAt should be 1 unit from camera
	mLookAt.x = mPos.x + forward.x;
	mLookAt.y = mPos.y + forward.y;
	mLookAt.z = mPos.z + forward.z;

	float3 zero;
	zero.x = 1;
	zero.y = 0;
	zero.z = 0;
	mYAngle = getAngleBetween(zero, forward);
	mXAngle = 0;

}
Camera::Camera(float3 pos, float3 lookAt, float3 up) {
	mPos = pos;
	mLookAt = lookAt;
	mUp = up;
	normalize(mUp.x, mUp.y, mUp.z);

	float3 forward;
	forward.x = mLookAt.x - mPos.x;
	forward.y = mLookAt.y - mPos.y;
	forward.z = mLookAt.z - mPos.z;
	normalize(forward.x, forward.y, forward.z);


	float3 zero;
	zero.x = 1;
	zero.y = 0;
	zero.z = 0;
	mYAngle = getAngleBetween(zero, forward);
	mXAngle = 0;

	// lookAt should be 1 unit from camera
	mLookAt.x = mPos.x + forward.x;
	mLookAt.y = mPos.y + forward.y;
	mLookAt.z = mPos.z + forward.z;

}
Camera::~Camera() {

}


void Camera::setPos(float3 pos) {
	mPos = pos;
}

void Camera::setLookAt(float3 lookAt) {
	mLookAt = lookAt;
}
void Camera::setUp(float3 up) {
	mUp = up;
}

float3 Camera::getPos() {
	return mPos;

}
float3 Camera::getLookAt() {
	return mLookAt;
}
float3 Camera::getUp() {
	return mUp;
}


// move functions
void Camera::moveX(float dx) {
	mPos.x += dx;
	mLookAt.x += dx;
}
void Camera::moveY(float dy) {
	mPos.y += dy;
	mLookAt.y += dy;

}
void Camera::moveZ(float dz) {
	mPos.z += dz;
	mLookAt.z += dz;

}

void Camera::moveForward(float dz) {
	float3 forward;
	forward.x = mLookAt.x - mPos.x;
	forward.y = mLookAt.y - mPos.y;
	forward.z = mLookAt.z - mPos.z;
	normalize(forward.x, forward.y, forward.z);
	
	mPos.x += forward.x*dz;
	mPos.y += forward.y*dz;
	mPos.z += forward.z*dz;
	mLookAt.x += forward.x*dz;
	mLookAt.y += forward.y*dz;
	mLookAt.z += forward.z*dz;
}

void Camera::moveSideways(float dz) {
	float3 forward;
	forward.x = mLookAt.x - mPos.x;
	forward.y = mLookAt.y - mPos.y;
	forward.z = mLookAt.z - mPos.z;
	normalize(forward.x, forward.y, forward.z);
	
	float3 right = crossProduct(forward, mUp);

	mPos.x += right.x*dz;
	mPos.y += right.y*dz;
	mPos.z += right.z*dz;
	mLookAt.x += right.x*dz;
	mLookAt.y += right.y*dz;
	mLookAt.z += right.z*dz;
}
// look at functions
void Camera::pitch(float dy) {

	mLookAt.y += dy*.1f;

}
void Camera::rotateY(float dy) {
	mYAngle += dy;

	mLookAt.x = mPos.x + cos(mYAngle);
	mLookAt.z = mPos.z + sin(mYAngle);

}

void Camera::rotateZ(float dz) {

}