#include "Mesh.h"

// color palette
float REDS[] = { 1.0f, 0.2f, 0.0f, .8f, .8f};
float GREENS[] = { 0.0f, 0.8f, 0.0f, 0, .8f};
float BLUES[] = { 0.0f, 0.2f, 1.0f, .8f, 0};

Mesh::Mesh(KFbxNode *node) {
	mPolys.resize(0);
//	mParent = parent;
//	mChildren.resize(0);
	extractVerts(node);
	const char *name = node->GetName();
	size_t length = strlen(name);
	mName = new char[length+1];
	strcpy(mName, name);
	setMatrix(mMatrix, node->GetGlobalFromCurrentTake(NULL));
	// pre-process vertices according to initial matrix
	transformVertices();
}

Mesh::~Mesh() {

}

void Mesh::extractVerts(KFbxNode *node) {
	KFbxMesh *fbxMesh = node->GetMesh();
	KFbxVector4 *controlPoints = fbxMesh->GetControlPoints();
	int numPolys = fbxMesh->GetPolygonCount();

	// not a mesh
	if (!fbxMesh) {
		return;
	}

	// material information
/*	KFbxLayerElementMaterial* layerMaterial= ((KFbxMesh*)node->GetNodeAttribute())->GetLayer(0)->GetMaterials();

	if (layerMaterial && layerMaterial->GetReferenceMode() == KFbxLayerElement::eINDEX) {
		if (layerMaterial->GetDirectArray().GetCount() > 0) {
			KFbxPropertyDouble3 lKFbxDouble3;
			KFbxPropertyDouble1 lKFbxDouble1;
			KFbxColor theColor;

			for (int j = 0; j < layerMaterial->GetDirectArray().GetCount(); j++)
			{
				KFbxSurfaceMaterial *material = layerMaterial->GetDirectArray().GetAt(j);

				// lambert material
				if(material->GetNewFbxClassId().Is(KFbxSurfaceLambert::ClassId)) {
											// Display the Diffuse Color
					lKFbxDouble3 =((KFbxSurfaceLambert *)material)->GetDiffuseColor();
					theColor.Set(lKFbxDouble3.Get()[0], lKFbxDouble3.Get()[1], lKFbxDouble3.Get()[2]);
				}
			}
		}
	}
*/

	static int colorIndex = 0;
	float red = REDS[colorIndex];
	float green = GREENS[colorIndex];
	float blue = BLUES[colorIndex];
	colorIndex = (colorIndex++) % 3;

	for (int i = 0; i < numPolys; i++) {

		int polygonSize = fbxMesh->GetPolygonSize(i);

		Vertex* verts = new Vertex[polygonSize];
		for (int j = 0; j < polygonSize; j++) {
			int cntrlPtIndex = fbxMesh->GetPolygonVertex(i, j);

			KFbxVector4 normal;
			fbxMesh->GetPolygonVertexNormal(i, j, normal);

			float x = (float)controlPoints[cntrlPtIndex][0];
			float y = (float)controlPoints[cntrlPtIndex][1];
			float z = (float)controlPoints[cntrlPtIndex][2];

			float nx = (float)normal[0];
			float ny = (float)normal[1];
			float nz = (float)normal[2];


			verts[j].x = x;
			verts[j].y = y;
			verts[j].z = -z;
			verts[j].nx = nx;
			verts[j].ny = ny;
			verts[j].nz = nz;
		}

		Polygon p;
		p.vertexCount = polygonSize;
		p.vertices = verts;
		p.r = red;
		p.g = green;
		p.b = blue;
		if (ENABLE_REFLECTION)
			p.reflection = .5f;
		else 
			p.reflection = 0.0f;

		if (ENABLE_REFRACTION)
			p.refraction = 0.5f;
		else 
			p.refraction = 0.0f;
		p.indexOfRefraction = 1.2f;
		fixNormals(&p);
		mPolys.push_back(p);

		for (int l = 0; l < fbxMesh->GetLayerCount(); l++) {
			//KFbxLayerElementPolygonGroup *group = fbxMesh->GetLayer(l)->GetPolygonGroups();

		}

	}
}


vector<Polygon> Mesh::getPolygons() {
	return mPolys;
}
/*
void Mesh::addChild(Mesh *c) {
	mChildren.push_back(c);
}

vector<Mesh *> Mesh::getChildren() {
	return mChildren;
}
*/
char *Mesh::getName() {
	return mName;
}

Matrix *Mesh::getMatrix() {
	return &mMatrix;
}
// transform vertices according to this mesh's matrix
void Mesh::transformVertices() {
	for (unsigned int i = 0; i < mPolys.size(); i++) {
		for (int j = 0; j < mPolys[i].vertexCount; j++) {
			float w = 1.0f;
			float x=mPolys[i].vertices[j].nx;
			float y=mPolys[i].vertices[j].ny;
			float z=mPolys[i].vertices[j].nz;
			float tx=x,ty=y,tz=z;

			multiplyMatrixVector(mMatrix, mPolys[i].vertices[j].x, mPolys[i].vertices[j].y, mPolys[i].vertices[j].z, w);

			w = 0.0f;
			multiplyMatrixVector(mMatrix, tx, ty, tz, w);
			normalize(tx, ty, tz);

		}
		for (int j = 0; j < mPolys[i].vertexCount; j++) {
			float w = 0.0f;
			
			multiplyMatrixVector(mMatrix, mPolys[i].vertices[j].nx, mPolys[i].vertices[j].ny, mPolys[i].vertices[j].nz, w);
			normalize(mPolys[i].vertices[j].nx, mPolys[i].vertices[j].ny, mPolys[i].vertices[j].nz);
		}
		fixNormals(&mPolys[i]);
	}

}

void setMatrix(Matrix &out, KFbxXMatrix in) {
	KFbxVector4 t, s, r;

	t = in.GetT();
	s = in.GetS();
	r = in.GetR();

	// fbx Z is opposite of mine
	t[2] = -t[2];
	
	identityMatrix(out);
	Matrix rot, trans, scale;
	identityMatrix(rot);
	identityMatrix(trans);
	identityMatrix(scale);

	translateMatrix(trans, (float)t[0], (float)t[1], (float)t[2]);
	scaleMatrix(scale, (float)s[0], (float)s[1], (float)s[2]);
	rotateMatrix(rot, -toRadians((float)r[0]), -toRadians((float)r[1]), toRadians((float)r[2]));

	Matrix temp;
	identityMatrix(temp);

	multiplyMatrix(temp, trans, rot);
	multiplyMatrix(out, temp, scale);

//	rotateZMatrix(out, 0.01f);
/*
out.c41 = (float)t[0];
	out.c42 = (float)t[1];
	out.c43 = (float)t[2];
	
	out.c11 = (float)s[0];
	out.c22 = (float)s[1];
	out.c33 = (float)s[2];

	out.c14=(float)r[0];
	out.c24=(float)r[1];
	out.c34=(float)r[2];
	out.c44=(float)r[3];
*/
	out.c44=1.0f;



/*	out.c11 = (float)in.Get(1, 1);
	out.c12 = (float)in.Get(1, 2);
	out.c13 = (float)in.Get(1, 3);
	out.c14 = (float)in.Get(1, 4);

	out.c21 = (float)in.Get(2, 1);
	out.c22 = (float)in.Get(2, 2);
	out.c23 = (float)in.Get(2, 3);
	out.c24 = (float)in.Get(2, 4);

	out.c31 = (float)in.Get(3, 1);
	out.c32 = (float)in.Get(3, 2);
	out.c33 = (float)in.Get(3, 3);
	out.c34 = (float)in.Get(3, 4);

	out.c41 = (float)in.Get(4, 1);
	out.c42 = (float)in.Get(4, 2);
	out.c43 = (float)in.Get(4, 3);
	out.c44 = (float)in.Get(4, 4);
*/

}
