#include <math.h>
#include "Vertex.h"
#include "Util.h"

void vertexCopy(Vertex &out, Vertex &in) {
	out.x = in.x;
	out.y = in.y;
	out.z = in.z;
	out.u = in.u;
	out.v = in.v;
	out.nx = in.nx;
	out.ny = in.ny;
	out.nz = in.nz;
}

void polygonCopy(Polygon &out, Polygon &in) {
	out.vertexCount = in.vertexCount;
	out.r = in.r;
	out.g = in.g;
	out.b = in.b;
	out.reflection = in.reflection;
	out.refraction = in.refraction;
	out.indexOfRefraction = in.indexOfRefraction;
	Vertex *v = new Vertex[out.vertexCount];
	for (int i = 0; i < out.vertexCount; i++) {
		vertexCopy(v[i], in.vertices[i]);
	}

	out.vertices = v;
}
// compute cross product
void cross(Vertex v[], float &xn, float &yn, float &zn) {
	float x1 = v[0].x - v[1].x;
	float y1 = v[0].y - v[1].y;
	float z1 = v[0].z - v[1].z;

	float x2 = v[1].x - v[2].x;
	float y2 = v[1].y - v[2].y;
	float z2 = v[1].z - v[2].z;

	normalize(x1, y1, z1);
	normalize(x2, y2, z2);

	xn = y1*z2 - z1*y2;
	yn = z1*x2 - x1*z2;
	zn = x1*y2 - y1*x2;

	normalize(xn, yn, zn);
}

// "fix" vertex normals by making them perpendicular to polygon
void fixNormals(Polygon *p) {
	float xn, yn, zn;
	cross(p->vertices, xn, yn, zn);

	for (int i = 0; i < p->vertexCount; i++) {
		p->vertices[i].nx = xn;
		p->vertices[i].ny = yn;
		p->vertices[i].nz = zn;
	}
}

